@@ -48,6 +48,7 @@ typedef struct
   struct _gpgme_op_keylist_result result;
 
   gpgme_key_t tmp_key;
+  /* This points to the last uid in tmp_key.  */
   gpgme_user_id_t tmp_uid;
   /* Something new is available.  */
   int key_cond;
@@ -63,8 +64,9 @@ release_op_data (void *hook)
 
   if (opd->tmp_key)
     gpgme_key_unref (opd->tmp_key);
-  if (opd->tmp_uid)
-    free (opd->tmp_uid);
+  /* opd->tmp_uid is actually part of opd->tmp_key, so we do not need
+     to release it here.  */
+
   while (key)
     {
       struct key_queue_item_s *next = key->next;
@@ -244,6 +246,10 @@ set_subkey_capability (gpgme_subkey_t subkey, const char *src)
 	  subkey->can_authenticate = 1;
 	  break;
 
+	case 'q':
+	  subkey->is_qualified = 1;
+	  break;
+
 	case 'd':
 	  subkey->disabled = 1;
 	  break;
@@ -292,6 +298,11 @@ set_mainkey_capability (gpgme_key_t key, const char *src)
 	case 'A':
 	  key->can_authenticate = 1;
 	  break;
+
+	case 'q':
+	case 'Q':
+	  key->is_qualified = 1;
+	  break;
         }
       src++;
     }
@@ -842,7 +853,7 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
   if (!ctx || !r_key || !fpr)
     return gpg_error (GPG_ERR_INV_VALUE);
   
-  if (strlen (fpr) < 16)	/* We have at least a key ID.  */
+  if (strlen (fpr) < 8)	/* We have at least a key ID.  */
     return gpg_error (GPG_ERR_INV_VALUE);
 
   /* FIXME: We use our own context because we have to avoid the user's
